#ifndef DIRUTILS_H
#define DIRUTILS_H

#include <stdio.h>
#include <ctype.h>
#include <string.h>


typedef bool (*directory_selector)(const char *, void*);

#ifdef WIN32

#include <io.h>
#include <windows.h>

#define FILE_SEPARATOR '\\'

typedef struct api_dir_iterator {
	char dir_name[256];
	char compare_to[256];
	long hfile;
	directory_selector select;
	void * arg;
	struct _finddata_t latest_file_data;
} API_DIR_ITERATOR;

#else

#include <dirent.h>

#define FILE_SEPARATOR '/'

typedef struct api_dir_iterator {
	char dir_name[256];
	char compare_to_upper[256];
	char compare_to_lower[256];
	directory_selector select;
	void * arg;
	DIR * dir;
	dirent * latest_file_data;
} API_DIR_ITERATOR;


#endif

class DirUtils
{
public:
	static char * OpenDirectory(const char * fname, API_DIR_ITERATOR & iterator,
		const char * comparison, directory_selector select, void * arg);
	static char * Next(API_DIR_ITERATOR & iterator);
	static void CloseDirectory(API_DIR_ITERATOR & iterator);
	
};



#endif

